package uk.ac.manchester.cs.owl.comp60462;

import org.semanticweb.owl.model.OWLOntology;
import org.semanticweb.owl.model.OWLOntologyManager;
import org.semanticweb.owl.model.OWLOntologyCreationException;
import org.semanticweb.owl.util.ToldClassHierarchyReasoner;
import org.semanticweb.owl.inference.OWLClassReasoner;
import org.semanticweb.owl.inference.OWLReasonerFactory;
import org.semanticweb.owl.inference.OWLReasoner;
import org.semanticweb.owl.inference.OWLReasonerException;
import org.semanticweb.reasonerfactory.pellet.PelletReasonerFactory;

import java.net.URI;
import java.util.Collections;
/*
 * Copyright (C) 2009, University of Manchester
 *
 * Modifications to the initial code base are copyright of their
 * respective authors, or their employers as appropriate.  Authorship
 * of the modifications may be determined from the ChangeLog placed at
 * the end of this file.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

/**
 * Author: Matthew Horridge<br>
 * The University of Manchester<br>
 * Information Management Group<br>
 * Date: 23-Mar-2009
 *
 * This class MUST HAVE A ZERO ARGUMENT CONSTRUCTOR!
 */
public class ShapeComputer {

    public static final URI CLASS_GRAPH_ONTOLOGY_PHYSICAL_URI = URI.create("http://www.cs.man.ac.uk/~bparsia/2009/comp60462/onts/cw1.owl");


    public ShapeComputer() {
        // Do not specify a different constructor to this empty constructor!
    }


    /**
     * Computes the asserted and inferred shape of a class graph in an ontology.
     * @param manager The manager that can be used to obtain a data factory etc.
     * @param ontology The ontology for which metrics are to be computed.
     * @param reasonerFactory A reasoner factory that should be used to create instances of a reasoner
     * @return An ontology that describes the shape of the class graph in the specified ontology.  This ontology
     * should specify the result in terms of the vocabulary in the classgraph.owl ontology which can be found
     * at the above physical URI (the result ontology should import the ontology class graph ontology).
     * @throws OWLReasonerException if there was a problem during reasoning
     * @throws OWLOntologyCreationException if there was a problem loading the class graph ontology
     */
    public OWLOntology computeShape(OWLOntologyManager manager, OWLOntology ontology) throws OWLReasonerException, OWLOntologyCreationException {

        // A reference to the class graph ontology which we need to import into our results ontology
        OWLOntology classGraphOntology = manager.loadOntologyFromPhysicalURI(CLASS_GRAPH_ONTOLOGY_PHYSICAL_URI);

        
        // The following code creates and sets up a reasoner
        OWLReasonerFactory reasonerFactory = new PelletReasonerFactory();
        OWLClassReasoner reasoner = reasonerFactory.createReasoner(manager);
        reasoner.loadOntologies(Collections.singleton(ontology));
        reasoner.classify();
        
        // The following code creates and sets up a "reasoner" that does no inference, simply acts on told information
	OWLClassReasoner toldReasoner = new ToldClassHierarchyReasoner(manager);
	toldReasoner.loadOntologies(Collections.singleton(ontology));
	toldReasoner.classify();
	
	// The two reasoners can then be used with whatever code you provide code to determine the shape. 
	
        // Compute the class graph shapes of the asserted and inferred class graphs
        // Return results as an ontology that imports the classGraphOntology
        throw new RuntimeException("TODO: Implement");
    }



}
